package choco;

import choco.real.exp.RealIntervalConstant;
import choco.real.RealInterval;


// **************************************************
// *                   J-CHOCO                      *
// *   Copyright (C) F. Laburthe, 1999-2003         *
// **************************************************
// *  an open-source Constraint Programming Kernel  *
// *     for Research and Education                 *
// **************************************************

/* File choco.Solution.java, last modified by flaburthe 21 oct. 2003 11:07:01 */

/**
 * A class storing a state of the problem
 */
public class Solution {
  /**
   * the problem owning the solution
   */
  protected Problem problem;

  /**
   * data storage for values of search variables
   */
  protected int[] intVarValues;

  protected RealInterval[] realVarValues;

  protected int[][] setVarValues;

  protected int objectiveValue;

  /**
   * Constructor
   * @param pb the problem owning the solution
   */
  public Solution(Problem pb) {
    problem = pb;
    int nbv = problem.getNbIntVars();
    intVarValues = new int[nbv];
    setVarValues = new int[problem.getNbSetVars()][];
    realVarValues = new RealInterval[problem.getNbRealVars()];
    for (int i = 0; i < nbv; i++) {
      intVarValues[i] = Integer.MAX_VALUE;
    }
    objectiveValue = Integer.MAX_VALUE;
  }

  public void recordIntValue(int intVarIndex, int intVarValue) {
    intVarValues[intVarIndex] = intVarValue;
  }

  public void recordSetValue(int setVarIndex, int[] setVarValue) {
    setVarValues[setVarIndex] = setVarValue;
  }

  public void recordRealValue(int realVarIndex, RealInterval realVarValue) {
    realVarValues[realVarIndex] = realVarValue;
  }
  public void recordIntObjective(int intObjectiveValue) {
    objectiveValue = intObjectiveValue;
  }

  public void restore() {
    try {
      for (int i = 0; i < intVarValues.length; i++) {
        if (intVarValues[i] != Integer.MAX_VALUE) {
          problem.getIntVar(i).setVal(intVarValues[i]);
        }
      }
      for (int i = 0; i < realVarValues.length; i++) {
          problem.getRealVar(i).intersect(realVarValues[i]);
      }
      for (int i = 0; i < setVarValues.length; i++) {
          problem.getSetVar(i).setVal(setVarValues[i]);
      }
      problem.propagate();
    } catch (ContradictionException e) {
      System.out.println("BUG in restoring solution !!!!!!!!!!!!!!!!");
      // TODO
    }
  }

  /**
   * Accessor to the value of a variable in a solution
   * @param varIndex the index of the variable among all variables of the problem
   * @return its value (whenever it is instantiated in the solution), or Integer.MAX_VALUE otherwise
   */
  public int getValue(int varIndex) {
    return intVarValues[varIndex];
  }

  public int[] getSetValue(int varIndex) {
    return setVarValues[varIndex];
  }

  public RealInterval getRealValue(int varIndex) {
    return realVarValues[varIndex];
  }
}
