// *********************************************
// *                   J-CHOCO                 *
// *   Copyright (c) F. Laburthe, 1999-2003    *
// *********************************************
// * Event-base contraint programming Engine   *
// *********************************************

// CVS Information
// File:               $RCSfile: CompositeConstraint.java,v $
// Version:            $Revision: 1.1.1.1 $
// Last Modification:  $Date: 2004/05/11 13:51:45 $
// Last Contributor:   $Author: laburthe $

package choco.bool;

import choco.*;
import choco.integer.*;

/**
 * An interface for all implementations of listeners using search variables.
 */
public interface CompositeConstraint extends Constraint, IntConstraint {

  /**
   * return the index of the subconstraint where the i-th variable is referenced
   *
   * @param varIdx the overall index of the variable (among all variables of the combination
   * @return the index of the subconstraint involving the variable
   *         or -1 if none can be found (which would be a definite bug !)
   */
  public int getSubConstraintIdx(int varIdx);

  /**
   * accessor to the sub-constraints from which the composite constraint is made of
   *
   * @param constIdx the index of the constraint
   * @return the appropriate subConstraint
   */
  public AbstractConstraint getSubConstraint(int constIdx);

  /**
   * returns the number of sub-constraints that the composite constraint is made of
   */
  public int getNbSubConstraints();

  /**
   * returns the global index of a variable within a constraint
   * @param subConstraint the subconstraint (a node in the composition tree)
   * @param localVarIdx the index of the variable, local to the subconstraint
   * @return the index of the variable in the global numbering associated to the composition (this)
   */
  public int getGlobalVarIndex(Constraint subConstraint, int localVarIdx);

}
