// *********************************************
// *                   J-CHOCO                 *
// *   Copyright (c) F. Laburthe, 1999-2003    *
// *********************************************
// * Event-base contraint programming Engine   *
// *********************************************

// CVS Information
// File:               $RCSfile: AbstractBinIntConstraint.java,v $
// Version:            $Revision: 1.2 $
// Last Modification:  $Date: 2004/06/30 11:12:26 $
// Last Contributor:   $Author: grochart $

package choco.integer.constraints;

import choco.ContradictionException;
import choco.Var;
import choco.AbstractVar;
import choco.bool.AbstractCompositeConstraint;
import choco.integer.constraints.AbstractIntConstraint;

// import choco.search.AbstractIntConstraint;

/**
 * An abstract class for all implementations of (binary) constraints over two search variable.
 */
public abstract class AbstractBinIntConstraint extends AbstractIntConstraint {

  /**
   * The first variable of the constraint.
   */

  public choco.integer.IntVar v0;


  /**
   * The second variable of the constraint.
   */

  public choco.integer.IntVar v1;


  /**
   * The index of the constraint among all listeners of its first variable.
   */

  public int cIdx0;


  /**
   * The index of the constraint among all listeners of its second variable.
   */

  public int cIdx1;


  /**
   * Let v be the i-th var of c, records that c is the n-th constraint involving v.
   */

  public void setConstraintIndex(int i, int val) {
    if (i == 0)
      cIdx0 = val;
    else if (i == 1)
      cIdx1 = val;
    else
      throw new Error("bug in setConstraintIndex i:" + i + " this: " + this);
  }


  /**
   * Returns the index of this constraint for the specified variables.
   */

  public int getConstraintIdx(int idx) {
    if (idx == 0)
      return cIdx0;
    else if (idx == 1)
      return cIdx1;
    else
      return -1;
  }


  /**
   * Checks if all the variables are instantiated.
   */

  public boolean isCompletelyInstantiated() {
    return (v0.isInstantiated() && v1.isInstantiated());
  }


  /**
   * Returns the number of varibles.
   */

  public int getNbVars() {
    return (2);
  }


  /**
   * Returns the specified variable.
   */

  public Var getVar(int i) {
    if (i == 0)
      return v0;
    else if (i == 1)
      return v1;
    else
      return null;
  }


  /**
   * Propagates on the constraint awake events.
   * @throws choco.ContradictionException
   */

  public void propagate() throws ContradictionException {
    awakeOnVar(0);
    awakeOnVar(1);
  }


  /**
   * Adds this constraint as new listeners for all the variables.
   */

  public void addConstraint() {
    setConstraintIndex(0, v0.addConstraint(this, 0, true));
    setConstraintIndex(1, v1.addConstraint(this, 1, true));
    if (this.hook != null) this.hook.addListener();
  }


  /**
   * Desactivate this listener from all the variables.
   */

  public void deactivateConstraint() {
    v0.deactivateConstraint(cIdx0);
    v1.deactivateConstraint(cIdx1);
    if (this.hook != null) this.hook.deactivateListener();
  }


  /**
   * Activates this listener in all the variables.
   */

  public void activateConstraint() {
    v0.activateConstraint(cIdx0);
    v1.activateConstraint(cIdx1);
    if (this.hook != null) this.hook.activateListener();
  }


  /**
   * Checks if the constraint is active.
   */

  public boolean isActive() {
    return (v0.isActive(cIdx0) || v1.isActive(cIdx1));
  }

  /**
   * Gets the <code>i</code>th search valued variable.
   */

  public choco.integer.IntVar getIntVar(int i) {
    if (i == 0)
      return v0;
    else if (i == 1)
      return v1;
    else
      return null;
  }

  public int assignIndices(AbstractCompositeConstraint root, int i) {
    int j=i;
    j++; root.connectVar((AbstractVar) v0, j);
    setConstraintIndex(0, v0.getNbConstraints() - 1);
    j++; root.connectVar((AbstractVar) v1, j);
    setConstraintIndex(1, v1.getNbConstraints() - 1);
    return j;
  }

}
