// *********************************************
// *                   J-CHOCO                 *
// *   Copyright (c) F. Laburthe, 1999-2003    *
// *********************************************
// * Event-base contraint programming Engine   *
// *********************************************

// CVS Information
// File:               $RCSfile: AbstractLargeIntConstraint.java,v $
// Version:            $Revision: 1.3 $
// Last Modification:  $Date: 2004/06/30 13:20:38 $
// Last Contributor:   $Author: grochart $

package choco.integer.constraints;

import choco.ContradictionException;
import choco.Var;
import choco.AbstractVar;
import choco.bool.AbstractCompositeConstraint;

//import choco.choco.*;
//import choco.search.AbstractIntConstraint;

/**
 * An abstract class for all implementations of listeners over many search variables.
 */
public abstract class AbstractLargeIntConstraint extends AbstractIntConstraint {

  /**
   * The list of variables of the constraint.
   */

  public choco.integer.IntVar[] vars;


  /**
   * The list, containing, for each variable, the index of the constraint among all
   * its incident listeners.
   */

  public int[] cIndices;

  /**
   * The search constant attached to the constraint.
   */

  public int cste;

  /**
   * Let <i>v</i> be the <i>i</i>-th var of <i>c</i>, records that <i>c</i> is the
   * <i>n</i>-th constraint involving <i>v</i>.
   */

  public void setConstraintIndex(int i, int val) {
    if (i >= 0 && i < vars.length)
      cIndices[i] = val;
    else
      throw new Error("bug in setConstraintIndex i:" + i + " this: " + this);
  }


  /**
   * Returns the index of the constraint in the specified variable.
   */

  public int getConstraintIdx(int i) {
    if (i >= 0 && i < vars.length)
      return cIndices[i];
    else
      return -1;
  }


  /**
   * Checks wether all the variables are instantiated.
   */

  public boolean isCompletelyInstantiated() {
    int nVariables = vars.length;
    for (int i = 0; i < nVariables; i++) {
      if (!(vars[i].isInstantiated()))
        return false;
    }
    return true;
  }


  /**
   * Returns the number of variables.
   */

  public int getNbVars() {
    return vars.length;
  }


  /**
   * Returns the <code>i</code>th variable.
   */

  public Var getVar(int i) {
    if (i >= 0 && i < vars.length)
      return vars[i];
    else
      return null;
  }


  /**
   * Propagates the constraint awake events.
   * @throws choco.ContradictionException
   */

  public void propagate() throws ContradictionException {
    for (int i = 0; i < vars.length; i++)
      awakeOnVar(i);
  }


  /**
   * Adds this constraint as new listeners in all the variables.
   */

  public void addConstraint() {
   for (int i = 0; i < vars.length; i++) {
      setConstraintIndex(i, vars[i].addConstraint(this, i, true));
   }
    if (this.hook != null) this.hook.addListener();
  }


  /**
   * Deativates this listener in all the variables.
   */

  public void deactivateConstraint() {
    for (int i = 0; i < vars.length; i++)
      vars[i].deactivateConstraint(cIndices[i]);
    if (this.hook != null) this.hook.deactivateListener();
  }


  /**
   * Activates this listener in variables.
   */

  public void activateConstraint() {
    for (int i = 0; i < vars.length; i++)
      vars[i].activateConstraint(cIndices[i]);
    if (this.hook != null) this.hook.activateListener();
  }


  /**
   * Checks if the constraint is active (e.g. plays a role in the propagation phase).
   * @return true if the constraint is indeed currently active
   */

  public boolean isActive() {
    for (int i = 0; i < vars.length; i++) {
      if (vars[i].isActive(cIndices[i]))
        return true;
    }
    return false;
  }

  /**
   * Gets the <code>i</code>th search valued variable.
   */

  public choco.integer.IntVar getIntVar(int i) {
    if (i >= 0 && i < getNbVars())
      return this.vars[i];
    else
      return null;
  }

  /**
   * constructor: allocates the data util for n variables
   * @param n the number of variables involved in the constraint
   */
  public AbstractLargeIntConstraint(int n) {
    vars = new choco.integer.var.IntDomainVar[n];
    cIndices = new int[n];
  }

  public int assignIndices(AbstractCompositeConstraint root, int i) {
    int j=i;
    for (int k=0; k<getNbVars(); k++) {
      j++;
      root.connectVar((AbstractVar) vars[k], j);
      setConstraintIndex(k, vars[k].getNbConstraints() - 1);
    }
    return j;
  }
}
