// *********************************************
// *                   J-CHOCO                 *
// *   Copyright (c) F. Laburthe, 1999-2003    *
// *********************************************
// * Event-base contraint programming Engine   *
// *********************************************

// CVS Information
// File:               $RCSfile: AbstractTernIntConstraint.java,v $
// Version:            $Revision: 1.2 $
// Last Modification:  $Date: 2004/06/30 11:12:26 $
// Last Contributor:   $Author: grochart $

package choco.integer.constraints;

import choco.ContradictionException;
import choco.Var;
import choco.AbstractVar;
import choco.bool.AbstractCompositeConstraint;

//import choco.choco.*;
//import choco.search.AbstractIntConstraint;

/**
 * An abstract class for all implementations of (unary) listeners over one
 * search variable.
 */
public abstract class AbstractTernIntConstraint extends AbstractIntConstraint {

  /**
   * The first variable of the constraint.
   */
  public choco.integer.IntVar v0;

  /**
   * The second variable of the constraint.
   */
  public choco.integer.IntVar v1;

  /**
   * The third variable of the constraint.
   */
  public choco.integer.IntVar v2;

  /**
   * The index of the constraint among all listeners of its first variable.
   */
  public int cIdx0;

  /**
   * The index of the constraint among all listeners of its second variable.
   */
  public int cIdx1;

  /**
   * The index of the constraint among all listeners of its third variable.
   */
  public int cIdx2;

  /**
   * Let v be the i-th var of c, records that c is the n-th constraint involving v.
   */
  public void setConstraintIndex(int i, int val) {
    if (i == 0)
      cIdx0 = val;
    else if (i == 1)
      cIdx1 = val;
    else if (i == 2)
      cIdx2 = val;
    else
      throw new Error("bug in setConstraintIndex i:" + i + " this: " + this);
  }

  /**
   * Returns the index of the constraint in the specified variable.
   */
  public int getConstraintIdx(int idx) {
    if (idx == 0)
      return cIdx0;
    else if (idx == 1)
      return cIdx1;
    else if (idx == 2)
      return cIdx2;
    else
      return -1;
  }

  /**
   * Checks if all the variables are instantiated.
   */
  public boolean isCompletelyInstantiated() {
    return (v0.isInstantiated() && v1.isInstantiated() && v2.isInstantiated());
  }

  /**
   * Returns the number of variables.
   */
  public int getNbVars() {
    return 3;
  }

  /**
   * Gets the specified variable.
   */
  public Var getVar(int i) {
    if (i == 0)
      return v0;
    else if (i == 1)
      return v1;
    else if (i == 2)
      return v2;
    else
      return null;
  }

  /**
   * Propagation for the constraint awake var.
   * @throws choco.ContradictionException
   */
  public void propagate() throws ContradictionException {
    awakeOnVar(0);
    awakeOnVar(1);
    awakeOnVar(2);
  }

  /**
   * Adds this constraint as a new listener for each variable.
   */
  public void addConstraint() {
    setConstraintIndex(0, v0.addConstraint(this, 0, true));
    setConstraintIndex(1, v1.addConstraint(this, 1, true));
    setConstraintIndex(2, v2.addConstraint(this, 2, true));
    if (this.hook != null) this.hook.addListener();
  }

  /**
   * Deactivates this listener in all the variables.
   */
  public void deactivateConstraint() {
    v0.deactivateConstraint(cIdx0);
    v1.deactivateConstraint(cIdx1);
    v2.deactivateConstraint(cIdx2);
    if (this.hook != null) this.hook.deactivateListener();
  }

  /***
   * Activates this listener in all the variables.
   */
  public void activateConstraint() {
    v0.activateConstraint(cIdx0);
    v1.activateConstraint(cIdx1);
    v2.activateConstraint(cIdx2);
    if (this.hook != null) this.hook.activateListener();
  }

  /**
   * Checks if the constraint is active.
   */
  public boolean isActive() {
    return (v0.isActive(cIdx0) || v1.isActive(cIdx1) || v2.isActive(cIdx2));
  }

  /**
   * Gets the <code>i</code>th search valued variable.
   */
  public choco.integer.IntVar getIntVar(int i) {
    if (i == 0)
      return v0;
    else if (i == 1)
      return v1;
    else if (i == 2)
      return v2;
    else
      return null;
  }

   public int assignIndices(AbstractCompositeConstraint root, int i) {
    int j=i;
    j++; root.connectVar((AbstractVar) v0, j);
    setConstraintIndex(0, v0.getNbConstraints() - 1);
    j++; root.connectVar((AbstractVar) v1, j);
    setConstraintIndex(1, v1.getNbConstraints() - 1);
    j++; root.connectVar((AbstractVar) v2, j);
    setConstraintIndex(2, v2.getNbConstraints() - 1);
    return j;
  }
}
