// *********************************************
// *                   J-CHOCO                 *
// *   Copyright (c) F. Laburthe, 1999-2003    *
// *********************************************
// * Event-base contraint programming Engine   *
// *********************************************

// CVS Information
// File:               $RCSfile: EqualXC.java,v $
// Version:            $Revision: 1.1.1.1 $
// Last Modification:  $Date: 2004/05/11 13:51:45 $
// Last Contributor:   $Author: laburthe $

package choco.integer.constraints;

import choco.ContradictionException;
import choco.AbstractConstraint;
import choco.Constraint;
import choco.integer.var.IntDomainVar;
import choco.integer.IntVar;

//import choco.search.AbstractUnIntConstraint;

/**
 * Implements a constraint X = C, with X a variable and C a constant.
 */
public class EqualXC extends AbstractUnIntConstraint {

  /**
   * The search constant of the constraint
   */
  protected final int cste;

  /**
   * Constructs the constraint with the specified variables and constant.
   *
   * @param x0 the search valued domain variable
   * @param c the search constant used in the equality.
   */

  public EqualXC(choco.integer.IntVar x0, int c) {
    this.v0 = (choco.integer.var.IntDomainVar) x0;
    this.cste = c;
  }

  /**
   * The one and only propagation method. <br>
   * Note that after the first propagation, the constraint could be set passive
   * (to prevent from further calls to propagation methods), but this management seems too heavy to be worthwhile
   */

  public void propagate() throws ContradictionException {
    logger.finest("VAL(" + v0.toString() + ") = " + this.cste);
    v0.instantiate(this.cste, this.cIdx0);
  }

  public void awakeOnVar(int idx) throws ContradictionException {
    logger.finest("VAL(" + v0.toString() + ") = " + this.cste);
    v0.instantiate(this.cste, this.cIdx0);
  }

  public void awakeOnInf(int idx) throws ContradictionException {
    logger.finest("VAL(" + v0.toString() + ") = " + this.cste);
    v0.instantiate(this.cste, this.cIdx0);
  }

  public void awakeOnSup(int idx) throws ContradictionException {
    logger.finest("VAL(" + v0.toString() + ") = " + this.cste);
    v0.instantiate(this.cste, this.cIdx0);
  }

  public void awakeOnInst(int idx) throws ContradictionException {
    logger.finest("VAL(" + v0.toString() + ") = " + this.cste);
    v0.instantiate(this.cste, this.cIdx0);
  }

  public void awakeOnRem(int idx, int x) throws ContradictionException {
    logger.finest("VAL(" + v0.toString() + ") = " + this.cste);
    v0.instantiate(this.cste, this.cIdx0);
  }


  /**
   * Checks if the listeners must be checked or must fail.
   */

  public Boolean isEntailed() {
    if ((v0.getInf() == v0.getSup()) &&
        (v0.getInf() == this.cste))
      return Boolean.TRUE;
    else if ((v0.getSup() < this.cste) ||
        (v0.getInf() > this.cste))
      return Boolean.FALSE;
    else
      return null;
  }

  /**
   * Checks if the constraint is satisfied when the variables are instantiated.
   */

  public boolean isSatisfied() {
    return (v0.getValue() == this.cste);
  }

  /**
   * tests if the constraint is consistent with respect to the current state of domains
   * @return true iff the constraint is bound consistent (same as arc consistent)
   */
  public boolean isConsistent() {
    return (v0.isInstantiatedTo(this.cste));
  }

  public AbstractConstraint opposite() {
    return (AbstractConstraint) getProblem().neq(v0, cste);
    // return NotEqualXC(v0, cste);
  }

  public final boolean isEquivalentTo(Constraint compareTo) {
    if (compareTo instanceof EqualXC) {
      EqualXC c = (EqualXC) compareTo;
      return ( (this.v0 == c.v0) && (this.cste == c.cste) );
    } else {
      return false;
    }
  }
  
}
