// *********************************************
// *                   J-CHOCO                 *
// *   Copyright (c) F. Laburthe, 1999-2003    *
// *********************************************
// * Event-base contraint programming Engine   *
// *********************************************

// CVS Information
// File:               $RCSfile: StoredVector.java,v $
// Version:            $Revision: 1.1.1.1 $
// Last Modification:  $Date: 2004/05/11 13:51:47 $
// Last Contributor:   $Author: laburthe $

package choco.mem;

import java.util.*;
import java.util.logging.Logger;

/**
 * StoredVector implement backtrackable dynamic arrays, without using time-stamping.
 *
 * TODO: Should certainly be reimplemented with an array.
 */
public class StoredVector implements List {
  /**
   * Reference to an object for logging trace statements related memory & backtrack (using the java.util.logging package)
   */

  private static Logger logger = Logger.getLogger("choco.mem");

  /**
   * The current values of the vector
   */
  private Vector entries;

  /**
   * The current number of values stored in the vector
   */
  private StoredInt nEntries;


  /**
   * The main environment object handling all memory management issues
   */
  public Environment environment;

  /**
   * The trailing stack holding updates to such {@link StoredVector}
   */
  private final StoredVectorTrail trail;

  /**
   * Constructor
   * @param env The main object handling all memory/backtrack management issues
   */
  public StoredVector(Environment env) {
    environment = env;
    entries = new Vector();
    nEntries = new StoredInt(env, 0);
    trail = (StoredVectorTrail) this.environment.getTrail(Environment.VECTOR_TRAIL);
  }

  /**
   * accessor: read one entry from the {@link StoredVector}
   * @param index the index in the vector
   * @return the value contained at that index
   */
  public Object get(int index) {
    if (index < nEntries.get() && index >= 0)
      return entries.get(index);
    else
      return null;
  }

  /**
   * modifying an entry storing if needed the former value on the trailing stack
   * @param o the new entry
   * @param index the index in the vector
   * */
  public Object set(int index, Object o) {
    // assert (index <= size.get())
    Object formerValue = entries.get(index);
    if (o != formerValue)
      trail.notifyUpdate(this, index, formerValue);
    //((StoredVectorTrail)this.environment.
    //                  getTrail(Environment.VECTOR_TRAIL)).
    //                  notifyUpdate(this, index, formerValue);
    //environment.notifyUpdate(this, index, formerValue);
    entries.set(index, o);
    return formerValue;
  }

  /**
   * modifying an entry without storing the former value on the trailing stack
   * @param o the new entry
   * @param index the index in the vector
   * */
  protected void _set(int index, Object o) {
    entries.set(index, o);
  }

  /**
   * Adding an object add the end ofs the {@link StoredVector}
   * @param o the object to be added
   * @return always true
   */
  public boolean add(Object o) {
    entries.add(o);
    int n = nEntries.get();
    nEntries.set(n + 1);
    return true;
  }

  /**
   * Adding an objet at a given index of a vector, and shifting all object after it, one index further
   * @param index the index where the object is to be inserted
   * @param o the object to be inserted
   */
  public void add(int index, Object o) {
    // TODO
  }

  /**
   * Testing if an object is contained in a {@link StoredVector}
   * @param o the object that is being looked for
   * @return true in case the object is present
   */
  public boolean contains(Object o) {
    int idx = entries.indexOf(o);
    if (idx >= 0) {
      if (idx <= nEntries.get())
        return true;
      else
        return false;
    } else
      return false;
  }

  public boolean equals(Object o) {
    // TODO
    return false;
  }

  public int hashCode() {
    // TODO
    return entries.hashCode();
  }

  public int indexOf(Object o) {
    int idx = entries.indexOf(o);
    if (idx >= 0) {
      if (idx <= nEntries.get())
        return idx;
      else
        return -1;
    } else
      return -1;
  }

  public int lastIndexOf(Object o) {
    // TODO
    return -1;
  }

  public boolean isEmpty() {
    return (nEntries.get() == 0);
  }

  public Iterator iterator() {
    // TODO
    return null;
  }

  public ListIterator listIterator() {
    // TODO
    return null;
  }

  public ListIterator listIterator(int index) {
    // TODO
    return null;
  }

  public Object remove(int index) {
    // TODO
    return null;
  }

  public boolean remove(Object o) {
    // TODO
    return true;
  }

  public int size() {
    return nEntries.get();
  }

  public List subList(int fromIndex, int toIndex) {
    return entries.subList(fromIndex, toIndex);
  }

  public Object[] toArray() {
    return entries.subList(0, nEntries.get()).toArray();
  }

  public Object[] toArray(Object[] a) {
    return entries.subList(0, nEntries.get()).toArray(a);
  }

  public boolean addAll(Collection c) {
    // TODO
    return true;
  }

  public boolean addAll(int index, Collection c) {
    // TODO
    return true;
  }

  public void clear() {
    // TODO
  }

  public boolean containsAll(Collection c) {
    // TODO
    return true;
  }

  public boolean removeAll(Collection c) {
    // TODO
    return true;
  }

  public boolean retainAll(Collection c) {
    // TODO
    return true;
  }
}