//     ___  ___         PaLM library for Choco system
//    /__ \/ __\        (c) 2001 - 2004 -- Narendra Jussien
//   ____\ |/_____
//  /_____\/_____ \     Explanation based constraint programming tool
// |/   (_)(_)   \|
//       \ /            Version 0.1
//       \ /            January 2004
//       \ /
// ______\_/_______     Contibutors: Franois Laburthe, Hadrien Cambazard, Guillaume Rochart...

package choco.palm.integer.constraints;

import choco.palm.explain.PalmConstraintPlugin;
import choco.palm.integer.AbstractPalmUnIntConstraint;
import choco.palm.integer.PalmIntVar;
import choco.palm.integer.PalmIntDomain;
import choco.palm.PalmProblem;

import java.util.Set;

import choco.integer.IntVar;

/**
 * Created by IntelliJ IDEA.
 * User: grochart
 * Date: 26 ao?t 2003
 * Time: 09:18:02
 * To change this template use Options | File Templates.
 */
public class PalmNotEqualXC extends AbstractPalmUnIntConstraint {
  protected final int cste;

  public PalmNotEqualXC(IntVar v0, int cste) {
    this.v0 = v0;
    this.cste = cste;
    this.hook = new PalmConstraintPlugin(this);
  }

  public String toString() {
    return this.v0 + " !== " + this.cste;
  }

  public void propagate() {
    if (this.v0.hasEnumeratedDomain()) {
      choco.palm.explain.Explanation expl = ((PalmProblem) this.getProblem()).makeExplanation();
      ((PalmConstraintPlugin) this.getPlugIn()).self_explain(expl);
      ((PalmIntVar) this.v0).removeVal(this.cste, this.cIdx0, expl);
    } else {
      if (this.v0.getInf() == this.cste) {
        choco.palm.explain.Explanation expl = ((PalmProblem) this.getProblem()).makeExplanation();
        ((PalmConstraintPlugin) this.getPlugIn()).self_explain(expl);
        ((PalmIntVar) this.v0).updateInf(this.cste + 1, this.cIdx0, expl);
      }
      if (this.v0.getSup() == this.cste) {
        choco.palm.explain.Explanation expl = ((PalmProblem) this.getProblem()).makeExplanation();
        ((PalmConstraintPlugin) this.getPlugIn()).self_explain(expl);
        ((PalmIntVar) this.v0).updateSup(this.cste - 1, this.cIdx0, expl);
      }
    }
  }

  public void awakeOnInf(int idx) {
    if (this.v0.getInf() == this.cste) {
      choco.palm.explain.Explanation expl = ((PalmProblem) this.getProblem()).makeExplanation();
      ((PalmConstraintPlugin) this.getPlugIn()).self_explain(expl);
      ((PalmIntVar) this.v0).updateInf(this.cste + 1, this.cIdx0, expl);
    }
  }

  public void awakeOnSup(int idx) {
    if (this.v0.getSup() == this.cste) {
      choco.palm.explain.Explanation expl = ((PalmProblem) this.getProblem()).makeExplanation();
      ((PalmConstraintPlugin) this.getPlugIn()).self_explain(expl);
      ((PalmIntVar) this.v0).updateInf(this.cste - 1, this.cIdx0, expl);
    }
  }

  public void awakeOnRem() {
  }

  public void awakeOnRestoreInf(int idx) {
    this.awakeOnInf(idx);
  }

  public void awakeOnRestoreSup(int idx) {
    this.awakeOnSup(idx);
  }

  public void awakeOnRestoreVal(int idx, int val) {
    if (this.cste == val) {
      choco.palm.explain.Explanation expl = ((PalmProblem) this.getProblem()).makeExplanation();
      ((PalmConstraintPlugin) this.getPlugIn()).self_explain(expl);
      ((PalmIntVar) this.v0).removeVal(val, this.cIdx0, expl);
    }
  }

  public Boolean isEntailed() {
    if (this.v0.isInstantiatedTo(this.cste))
      return Boolean.FALSE;
    else if (!(this.v0.canBeInstantiatedTo(this.cste))) return Boolean.TRUE;
    return null;
  }

  public boolean isSatisfied() {
    return this.v0.getValue() != this.cste;
  }

  public Set whyIsTrue() {
    choco.palm.explain.Explanation expl = ((PalmProblem) this.getProblem()).makeExplanation();
    ((PalmIntVar) this.v0).self_explain(PalmIntDomain.VAL, this.cste, expl);
    return expl.toSet();
  }

  public Set whyIsFalse() {
    choco.palm.explain.Explanation expl = ((PalmProblem) this.getProblem()).makeExplanation();
    ((PalmIntVar) this.v0).self_explain(PalmIntDomain.DOM, expl);
    return expl.toSet();
  }
}
