//     ___  ___         PaLM library for Choco system
//    /__ \/ __\        (c) 2001 - 2004 -- Narendra Jussien
//   ____\ |/_____
//  /_____\/_____ \     Explanation based constraint programming tool
// |/   (_)(_)   \|
//       \ /            Version 0.1
//       \ /            January 2004
//       \ /
// ______\_/_______     Contibutors: Franois Laburthe, Hadrien Cambazard, Guillaume Rochart...

package choco.palm.integer.constraints;

import choco.palm.explain.PalmConstraintPlugin;
import choco.palm.explain.Explanation;
import choco.palm.integer.AbstractPalmBinIntConstraint;
import choco.palm.integer.PalmIntVar;
import choco.palm.integer.PalmIntDomain;
import choco.palm.PalmProblem;

import java.util.Set;
import java.util.logging.Logger;
import java.util.logging.Level;

import choco.ContradictionException;
import choco.integer.IntVar;

/**
 * Created by IntelliJ IDEA.
 * User: grochart
 * Date: 26 ao?t 2003
 * Time: 10:57:55
 * To change this template use Options | File Templates.
 */
public class PalmNotEqualXYC extends AbstractPalmBinIntConstraint {
  protected final int cste;

  public PalmNotEqualXYC(IntVar v0, IntVar v1, int cste) {
    this.v0 = v0;
    this.v1 = v1;
    this.cste = cste;
    this.hook = new PalmConstraintPlugin(this);
  }

  public String toString() {
    return this.v0 + " !== " + this.v1 + " + " + this.cste;
  }

  public void propagate() throws ContradictionException {
    if (this.v0.hasEnumeratedDomain() && this.v1.hasEnumeratedDomain()) {
      if (this.v0.getDomainSize() == 1) {
        choco.palm.explain.Explanation expl = ((PalmProblem) this.getProblem()).makeExplanation();
        ((PalmConstraintPlugin) this.getPlugIn()).self_explain(expl);
        ((PalmIntVar) this.v0).self_explain(PalmIntDomain.DOM, expl);
        ((PalmIntVar) this.v1).removeVal(((PalmIntVar) this.v0).getInf() - this.cste, this.cIdx1, expl);
      }
      if (this.v1.getDomainSize() == 1) {
        choco.palm.explain.Explanation expl = ((PalmProblem) this.getProblem()).makeExplanation();
        ((PalmConstraintPlugin) this.getPlugIn()).self_explain(expl);
        ((PalmIntVar) this.v1).self_explain(PalmIntDomain.DOM, expl);
        ((PalmIntVar) this.v0).removeVal(((PalmIntVar) this.v1).getInf() + this.cste, this.cIdx0, expl);
      }
    } else {
      if (this.v0.getInf() == this.v0.getSup()) {
        choco.palm.explain.Explanation expl = ((PalmProblem) this.getProblem()).makeExplanation();
        ((PalmConstraintPlugin) this.getPlugIn()).self_explain(expl);
        ((PalmIntVar) this.v0).self_explain(PalmIntDomain.DOM, expl);
        if (this.v1.getInf() == this.v0.getInf() - this.cste) {
          ((PalmIntVar) this.v1).updateInf(this.v0.getInf() - this.cste + 1, this.cIdx1, expl);
        }
        if (this.v1.getSup() == this.v0.getInf() - this.cste) {
          ((PalmIntVar) this.v1).updateSup(this.v0.getInf() - this.cste - 1, this.cIdx1, (Explanation) expl.copy());
        }
      } else if (this.v1.getInf() == this.v1.getSup()) {
        choco.palm.explain.Explanation expl = ((PalmProblem) this.getProblem()).makeExplanation();
        ((PalmConstraintPlugin) this.getPlugIn()).self_explain(expl);
        ((PalmIntVar) this.v1).self_explain(PalmIntDomain.DOM, expl);
        if (this.v0.getInf() == this.v1.getInf() + this.cste) {
          ((PalmIntVar) this.v0).updateInf(this.v1.getInf() + this.cste + 1, this.cIdx0, expl);
        }
        if (this.v0.getSup() == this.v1.getInf() + this.cste) {
          ((PalmIntVar) this.v0).updateSup(this.v1.getInf() + this.cste - 1, this.cIdx0, (Explanation) expl.copy());
        }
      }
    }
  }

  public void awakeOnInf(int idx) {
    if ((idx == 0) && (this.v0.getInf() == this.v0.getSup())) {
      choco.palm.explain.Explanation expl = ((PalmProblem) this.getProblem()).makeExplanation();
      ((PalmConstraintPlugin) this.getPlugIn()).self_explain(expl);
      ((PalmIntVar) this.v0).self_explain(PalmIntDomain.DOM, expl);
      if (this.v1.getInf() == this.v0.getInf() - this.cste) {
        ((PalmIntVar) this.v1).updateInf(this.v0.getInf() - this.cste + 1, this.cIdx1, expl);
      }
      if (this.v1.getSup() == this.v0.getInf() - this.cste) {
        ((PalmIntVar) this.v1).updateSup(this.v0.getInf() - this.cste - 1, this.cIdx1, (Explanation) expl.copy());
      }
    } else if ((idx == 1) && (this.v1.getInf() == this.v1.getSup())) {
      choco.palm.explain.Explanation expl = ((PalmProblem) this.getProblem()).makeExplanation();
      ((PalmConstraintPlugin) this.getPlugIn()).self_explain(expl);
      ((PalmIntVar) this.v1).self_explain(PalmIntDomain.DOM, expl);
      if (this.v0.getInf() == this.v1.getInf() + this.cste) {
        ((PalmIntVar) this.v0).updateInf(this.v1.getInf() + this.cste + 1, this.cIdx0, expl);
      }
      if (this.v0.getSup() == this.v1.getInf() + this.cste) {
        ((PalmIntVar) this.v0).updateSup(this.v1.getInf() + this.cste - 1, this.cIdx0, (Explanation) expl.copy());
      }
    }
  }

  public void awakeOnSup(int idx) {
    this.awakeOnInf(idx);
  }

  public void awakeOnRestoreInf(int idx) {
    this.awakeOnInf(idx);
  }

  public void awakeOnRestoreSup(int idx) {
    this.awakeOnInf(idx);
  }

  public void awakeOnRem(int idx, int val) {
    if (idx == 0) {
      if (this.v0.getDomainSize() == 1) {
        choco.palm.explain.Explanation expl = ((PalmProblem) this.getProblem()).makeExplanation();
        ((PalmConstraintPlugin) this.getPlugIn()).self_explain(expl);
        ((PalmIntVar) this.v0).self_explain(PalmIntDomain.DOM, expl);
        ((PalmIntVar) this.v1).removeVal(((PalmIntVar) this.v0).getInf() - this.cste, this.cIdx1, expl);
      }
    } else {
      if (this.v1.getDomainSize() == 1) {
        choco.palm.explain.Explanation expl = ((PalmProblem) this.getProblem()).makeExplanation();
        ((PalmConstraintPlugin) this.getPlugIn()).self_explain(expl);
        ((PalmIntVar) this.v1).self_explain(PalmIntDomain.DOM, expl);
        ((PalmIntVar) this.v0).removeVal(((PalmIntVar) this.v1).getInf() + this.cste, this.cIdx0, expl);
      }
    }
  }

  public void awakeOnRestoreVal(int idx, int val) {
    //this.awakeOnRem(idx, val);
    if (idx == 0) {
      if (v1.isInstantiatedTo(val - cste)) {
        choco.palm.explain.Explanation expl = ((PalmProblem) this.getProblem()).makeExplanation();
        ((PalmConstraintPlugin) this.getPlugIn()).self_explain(expl);
        ((PalmIntVar) this.v1).self_explain(PalmIntDomain.DOM, expl);
        ((PalmIntVar) this.v0).removeVal(val - cste, this.cIdx0, expl);
      }
    } else {
      if (v0.isInstantiatedTo(val + cste)) {
        choco.palm.explain.Explanation expl = ((PalmProblem) this.getProblem()).makeExplanation();
        ((PalmConstraintPlugin) this.getPlugIn()).self_explain(expl);
        ((PalmIntVar) this.v0).self_explain(PalmIntDomain.DOM, expl);
        ((PalmIntVar) this.v1).removeVal(val + cste, this.cIdx1, expl);
      }
    }
  }

  public Boolean isEntailed() {
    if ((this.v0.getSup() < this.v1.getInf() + this.cste) || (this.v1.getSup() < this.v0.getInf() - this.cste))
      return Boolean.TRUE;
    else if ((this.v0.getInf() == this.v0.getSup()) && (this.v1.getInf() == this.v1.getSup())
            && (this.v0.getInf() == this.v1.getInf() + this.cste))
      return Boolean.FALSE;
    return null;
  }

  public boolean isSatisfied() {
    return this.v0.getValue() !=  (this.v1.getValue() + this.cste);
  }

  public Set whyIsTrue() {
    if (Logger.getLogger("choco.palm").isLoggable(Level.WARNING))
      Logger.getLogger("choco.palm").warning("Not Yet implemented : NotEqual.whyIsTrue");
    return null;
  }

  public Set whyIsFalse() {
    if (Logger.getLogger("choco.palm").isLoggable(Level.WARNING))
      Logger.getLogger("choco.palm").warning("Not Yet implemented : NotEqual.whyIsFalse");
    return null;
  }
}
