package choco.palm.real;

import choco.real.RealModeler;
import choco.real.RealExp;
import choco.real.constraint.Equation;
import choco.real.exp.*;
import choco.Problem;
import choco.Constraint;
import choco.palm.real.constraints.PalmEquation;
import choco.palm.real.exp.PalmRealPlus;
import choco.palm.real.exp.PalmRealMinus;
import choco.palm.real.exp.PalmRealMult;
import choco.palm.real.exp.PalmRealIntervalConstant;

/**
 * J-CHOCO
 * Copyright (C) F. Laburthe, 1999-2003
 * <p/>
 * An open-source Constraint Programming Kernel
 * for Research and Education
 * <p/>
 * Created by: Guillaume on 24 juin 2004
 */
public class PalmRealModeler extends RealModeler {
  public PalmRealModeler(Problem pb) {
    super(pb);
  }

  /**
   * Makes an equation from an expression and a constantt interval. It is used by all methods building
   * constraints. This is  useful for subclassing this modeller  for another kind of problem (like PaLM).
   * @param exp The expression
   * @param cst The interval this expression should be in
   */
  protected Constraint makeEquation(RealExp exp, RealIntervalConstant cst) {
    return new PalmEquation(problem, exp, cst);
  }

  /**
   * Addition of two expressions.
   */
  public RealExp plus(RealExp exp1, RealExp exp2) {
    return new PalmRealPlus(problem, exp1, exp2);
  }

  /**
   * Substraction of two expressions.
   */
  public RealExp minus(RealExp exp1, RealExp exp2) {
    return new PalmRealMinus(problem, exp1, exp2);
  }

  /**
   * Multiplication of two expressions.
   */
  public RealExp mult(RealExp exp1, RealExp exp2) {
    return new PalmRealMult(problem, exp1, exp2);
  }

  /**
   * Power of an expression.
   */
  public RealExp power(RealExp exp, int power) {
    if (power == 2) {
      return mult(exp, exp);
    }
    throw new UnsupportedOperationException();
  }

  /**
   * Cosinus of an expression.
   */
  public RealExp cos(RealExp  exp) {
    throw new UnsupportedOperationException();
  }

  /**
   * Sinus of an expression.
   */
  public RealExp sin(RealExp exp) {
    throw new UnsupportedOperationException();
  }

  /**
   * Makes a constant interval from a double d ([d,d]).
   */
  public RealIntervalConstant cst(double d) {
    return new PalmRealIntervalConstant(d, d);
  }
}
