//     ___  ___         PaLM library for Choco system
//    /__ \/ __\        (c) 2001 - 2004 -- Narendra Jussien
//   ____\ |/_____
//  /_____\/_____ \     Explanation based constraint programming tool
// |/   (_)(_)   \|
//       \ /            Version 0.1
//       \ /            January 2004
//       \ /
// ______\_/_______     Contibutors: Franois Laburthe, Hadrien Cambazard, Guillaume Rochart...

package choco.palm.real;

import choco.real.var.RealVarImpl;
import choco.Problem;
import choco.ConstraintCollection;
import choco.Constraint;
import choco.palm.integer.PalmIntervalIntDomain;
import choco.palm.integer.PalmIntDomain;

import java.util.BitSet;

/**
 * Default implementation of PaLM real variables.
 */
public class PalmRealVarImpl extends RealVarImpl implements PalmRealVar {
  protected BitSet erasedCst = new BitSet();

  /**
   * Creates a variable with the specified bounds and the specified name.
   */
  public PalmRealVarImpl(Problem pb, String name, double a, double b) {
    super(pb, name, a, b);

    // Helps GC
    this.event = null;
    this.domain = null;

    // New Palm event and domain
    this.event = new PalmRealVarEvent(this);
    this.domain = new PalmRealDomainImpl(this, a, b);
  }

  /**
   * The name of the variable.
   */
  public String toString() {
    return name;
  }

  /**
   * Updates lower bound explanations: it removes not up-to-date explanations.
   */
  public void resetExplanationOnInf() {
    ((PalmRealDomain) this.domain).resetExplanationOnInf();
  }

  /**
   * Updates upper bound explanations: it removes not up-to-date explanations.
   */
  public void resetExplanationOnSup() {
    ((PalmRealDomain) this.domain).resetExplanationOnSup();
  }

  /**
   * Updates decision constraints on this variable: it removes all erased constraints.
   */
  public void updateDecisionConstraints() {
    ((PalmRealDomain) this.domain).updateDecisionConstraints();
  }

  /**
   * Lower bound of the domain should be restored to the specified value.
   */
  public void restoreInf(double newValue) {
    ((PalmRealDomain) this.domain).restoreInf(newValue);
  }

  /**
   * Upper bound of the domain should be restored to the specified value.
   */
  public void restoreSup(double newValue) {
    ((PalmRealDomain) this.domain).restoreSup(newValue);
  }

  /**
   * Merge explanation of the specified part of the domain to a constraint collection.
   */
  public void self_explain(int select, ConstraintCollection e) {
    ((PalmRealDomain) this.domain).self_explain(select, e);
  }

    /**
   * removes (permanently) a constraint from the list of constraints connected to the variable
   * @param constraintIdx
   */
  public void eraseConstraint(int constraintIdx) {
    constraints.set(constraintIdx, null);
    erasedCst.set(constraintIdx);
  }

  /**
   * Adds a new constraints, and makes it active if needed.
   */
  // TODO: this needs to become backtrackable
  public int addConstraint(Constraint c, int varIdx, boolean active) {
    int idx;
    if (erasedCst.length() == 0)  {
      constraints.add(c);
      indices.add(new Integer(varIdx));
      idx = constraints.size() - 1;
    } else {
      idx = erasedCst.nextSetBit(0);
      constraints.set(idx, c);
      indices.set(idx, new Integer(varIdx));
      erasedCst.clear(idx);
    }

    this.event.addListener(idx, active);

    return idx;
  }
}
