package choco.palm.real.constraints;

import choco.ContradictionException;
import choco.util.IntIterator;
import choco.palm.explain.Explanation;
import choco.palm.explain.PalmConstraintPlugin;
import choco.palm.PalmProblem;
import choco.palm.real.exp.PalmRealIntervalConstant;
import choco.palm.real.PalmRealVar;
import choco.palm.real.PalmRealInterval;
import choco.palm.integer.PalmIntVar;
import choco.integer.IntVar;
import choco.real.RealVar;
import choco.real.constraint.MixedCstElt;

import java.util.Set;

/**
 * Let x be an integer variable with n values and v be a real variable. Given n constant values a1 to an,
 * this constraint ensures that:
 *
 * <code>x = i iff v = ai</code>
 *
 * a1... an sequence is supposed to be ordered (a1&lt;a2&lt;... an)
 */
public class PalmMixedCstElt extends MixedCstElt implements PalmMixedConstraint {

  public PalmMixedCstElt(RealVar v0, IntVar v1, double[] values) {
    super(v0, v1, values);
    this.hook = new PalmConstraintPlugin(this);
  }

  public void awake() throws ContradictionException {
    // Ensures that integer domain is correct !
    Explanation e = ((PalmProblem) this.problem).makeExplanation();
    ((PalmConstraintPlugin)this.getPlugIn()).self_explain(e);
    ((PalmIntVar)intVar).updateSup(values.length - 1, intIdx, (Explanation) e.copy());
    ((PalmIntVar)intVar).updateInf(0, intIdx, e);
    this.propagate();
  }

  public void awakeOnRemovals(int varIdx, IntIterator deltaDomain) throws ContradictionException {
    this.propagate();
  }

  public void updateIInf() throws ContradictionException {
    Explanation e = ((PalmProblem) this.problem).makeExplanation();
    ((PalmRealVar)realVar).self_explain(PalmRealInterval.INF, e);
    ((PalmConstraintPlugin)this.getPlugIn()).self_explain(e);

    int inf = intVar.getInf();
    while (values[inf] < realVar.getInf()) {
      inf++;
    }
    ((PalmIntVar)intVar).updateInf(inf, intIdx, e);
  }

  public void updateISup() throws ContradictionException {
    Explanation e = ((PalmProblem) this.problem).makeExplanation();
    ((PalmRealVar)realVar).self_explain(PalmRealInterval.SUP, e);
    ((PalmConstraintPlugin)this.getPlugIn()).self_explain(e);

    int sup = intVar.getSup();
    while (values[sup] > realVar.getSup()) {
      sup--;
    }
    ((PalmIntVar)intVar).updateSup(sup, intIdx, e);
  }

  public void updateReal() throws ContradictionException {
    Explanation inf = ((PalmProblem)this.problem).makeExplanation();
    Explanation sup = ((PalmProblem)this.problem).makeExplanation();
    ((PalmIntVar)intVar).self_explain(PalmRealInterval.INF, inf);
    ((PalmConstraintPlugin)this.getPlugIn()).self_explain(inf);
    ((PalmIntVar)intVar).self_explain(PalmRealInterval.SUP, sup);
    ((PalmConstraintPlugin)this.getPlugIn()).self_explain(sup);
    realVar.intersect(new PalmRealIntervalConstant(values[intVar.getInf()], values[intVar.getSup()], inf, sup));
  }

  public void awakeOnRestoreInf(int idx) throws ContradictionException {
    if (idx == 0) {
      updateReal();
    } else {
      updateIInf();
      updateReal();
    }
  }

  public void awakeOnRestoreSup(int idx) throws ContradictionException {
    if (idx == 0) {
      updateReal();
    } else {
      updateISup();
      updateReal();
    }
  }

  public void awakeOnRestoreVal(int idx, int val) throws ContradictionException {
  }

  public void awakeOnRestoreVal(int idx, IntIterator it) throws ContradictionException {
    this.propagate();
  }

  public Set whyIsTrue() {
    return null;
  }

  public Set whyIsFalse() {
    return null;
  }

  public void updateDataStructuresOnConstraint(int idx, int select, int newValue, int oldValue) {
  }

  public void updateDataStructuresOnRestoreConstraint(int idx, int select, int newValue, int oldValue) {
  }
}
