//     ___  ___         PaLM library for Choco system
//    /__ \/ __\        (c) 2001 - 2004 -- Narendra Jussien
//   ____\ |/_____
//  /_____\/_____ \     Explanation based constraint programming tool
// |/   (_)(_)   \|
//       \ /            Version 0.1
//       \ /            January 2004
//       \ /
// ______\_/_______     Contibutors: Franois Laburthe, Hadrien Cambazard, Guillaume Rochart...

package choco.palm.real.exp;

import choco.real.exp.AbstractRealBinTerm;
import choco.real.RealExp;
import choco.real.RealInterval;
import choco.Problem;
import choco.ContradictionException;
import choco.ConstraintCollection;
import choco.palm.explain.Explanation;
import choco.palm.PalmProblem;
import choco.palm.prop.PalmEngine;
import choco.palm.real.PalmRealInterval;

/**
 * Abstract implementation of a real binary term (like addition for instance).
 */
public abstract class AbstractPalmRealBinTerm extends AbstractRealBinTerm implements PalmRealInterval {
  /**
   * Explanation of the last lower bound affectation.
   */
  protected Explanation explanationOnInf;

  /**
   * Explanation of the last upper bound affectation.
   */
  protected Explanation explanationOnSup;

  /**
   * Creates an abstract binary term with the two sub-expressions.
   */
  public AbstractPalmRealBinTerm(Problem pb, RealExp exp1, RealExp exp2) {
    super(pb, exp1, exp2);
    explanationOnInf  = ((PalmProblem) pb).makeExplanation();
    explanationOnSup  = ((PalmProblem) pb).makeExplanation();
  }

  /**
   * Updates the interval such that this interval is included in the interval parameter.
   * @param interval The interval this expression should be included in.
   * @param index The index of the constraint responsible of this reduction.
   * @throws ContradictionException
   */
  public void intersect(RealInterval interval, int index) throws ContradictionException {
    if (interval.getInf() > inf.get()) {
      inf.set(interval.getInf());
      explanationOnInf.empties();
      ((PalmRealInterval)interval).self_explain(INF, explanationOnInf);
    }
    if (interval.getSup() < sup.get()) {
      sup.set(interval.getSup());
      explanationOnSup.empties();
      ((PalmRealInterval)interval).self_explain(SUP, explanationOnSup);
    }
    if (inf.get() > sup.get()) {
      Explanation e = ((PalmProblem)this.getProblem()).makeExplanation();
      this.self_explain(DOM, e);
      ((PalmEngine)this.getProblem().getPropagationEngine()).raisePalmFakeContradiction(e);
    }
  }

  /**
   * Explains the state of this expression (lower/upper bounds or all domain)
   * @param select Specifies which part of the domain should be explained.
   * @param e Specifies the explanation in which these explaining constraints should be added.
   */
  public void self_explain(int select, ConstraintCollection e) {
    switch (select) {
      case INF: e.merge((ConstraintCollection) this.explanationOnInf); break;
      case SUP: e.merge((ConstraintCollection) this.explanationOnSup); break;
      case DOM:
        e.merge((ConstraintCollection) this.explanationOnInf);
        e.merge((ConstraintCollection) this.explanationOnSup);
        break;
    }
  }
}
