//     ___  ___         PaLM library for Choco system
//    /__ \/ __\        (c) 2001 - 2004 -- Narendra Jussien
//   ____\ |/_____
//  /_____\/_____ \     Explanation based constraint programming tool
// |/   (_)(_)   \|
//       \ /            Version 0.1
//       \ /            January 2004
//       \ /
// ______\_/_______     Contibutors: Franois Laburthe, Hadrien Cambazard, Guillaume Rochart...

package choco.palm.real.exp;

import choco.Problem;
import choco.ContradictionException;
import choco.palm.real.PalmRealInterval;
import choco.palm.real.PalmRealMath;
import choco.palm.PalmProblem;
import choco.real.RealExp;

/**
 * Implementation of a real multiplication expression.
 */
public class PalmRealMult extends AbstractPalmRealBinTerm {
  /**
   * Creates the product of two sub expressions.
   */
  public PalmRealMult(Problem pb, RealExp exp1, RealExp exp2) {
    super(pb, exp1, exp2);
  }

  /**
   * Tightens the value of the expressions, that is affects values with respect to the values
   * (and explanations) of sub expressions.
   */
  public void tighten() {
    PalmRealInterval res = PalmRealMath.mul((PalmProblem) this.getProblem(), exp1, exp2);
    inf.set(res.getInf());
    sup.set(res.getSup());
    explanationOnInf.empties(); res.self_explain(INF, explanationOnInf);
    explanationOnSup.empties(); res.self_explain(SUP, explanationOnSup);
  }

  /**
   * Projects current value on sub-expressions.
   * @throws ContradictionException
   */
  public void project() throws ContradictionException {
    PalmRealInterval res =
        PalmRealMath.odiv_wrt((PalmProblem) this.getProblem(), this, exp2, exp1);
    if (res.getInf() > res.getSup()) {  // TODO
      System.err.println("Palm Real  Mult...");
      problem.getPropagationEngine().raiseContradiction();
    }
    exp1.intersect(res);

    res = PalmRealMath.odiv_wrt((PalmProblem) this.getProblem(), this, exp1, exp2);
    if (res.getInf() > res.getSup()) {  // TODO
      System.err.println("Palm Real  Mult...");
      problem.getPropagationEngine().raiseContradiction();
    }
    exp2.intersect(res);
  }
}
