//     ___  ___         PaLM library for Choco system
//    /__ \/ __\        (c) 2001 - 2004 -- Narendra Jussien
//   ____\ |/_____
//  /_____\/_____ \     Explanation based constraint programming tool
// |/   (_)(_)   \|
//       \ /            Version 0.1
//       \ /            January 2004
//       \ /
// ______\_/_______     Contibutors: Franois Laburthe, Hadrien Cambazard, Guillaume Rochart...

package choco.palm.search;

import choco.palm.PalmProblem;
import choco.palm.search.PalmGlobalSearchSolver;
import choco.Var;
import choco.Problem;
import choco.Solution;
import choco.integer.IntVar;

import java.util.List;
import java.util.Iterator;

/**
 * Solution for a Palm problem.
 */

public class PalmSolution extends Solution {

  /**
   * Statistics for this solution (CPU, extensiions...).
   */

  protected int[] lstat;


  /**
   * Creates a solution for the specified problem.
   */

  public PalmSolution(Problem pb) {
    super(pb);
    this.lstat = new int[((PalmGlobalSearchSolver)pb.getSolver().getSearchSolver()).getNbLimit()];
  }


  public void recordStatistic(int statIndex, int statValue) {
    lstat[statIndex] = statValue;
  }


  /**
   * Returns a statistic about the solution.
   * @param stat The statitic constant (<code>PalmProblem.RLX/EXT/CPU</code>).
   */

  public int getStatistic(int stat) {
    return lstat[stat];
  }

  /**
   * Returns the value of the variable <code>idx</code>.
   * @param idx the variable number
   * @return the value of the variable
   */

  public int getValue(int idx) {    // TODO : dans Choco ?
    return this.intVarValues[idx];
  }


  /**
   * Pretty display of the solution.
   */

  public String toString() {
    StringBuffer buf = new StringBuffer();

    buf.append("SOL ");
    for (int i = 0; i < this.problem.getNbIntVars(); i++) {
      IntVar var = (IntVar) this.problem.getIntVar(i);
      buf.append(var + ":" + this.intVarValues[i] + " \n");
    }

    return buf.toString();
  }
}
