//     ___  ___         PaLM library for Choco system
//    /__ \/ __\        (c) 2001 - 2004 -- Narendra Jussien
//   ____\ |/_____
//  /_____\/_____ \     Explanation based constraint programming tool
// |/   (_)(_)   \|
//       \ /            Version 0.1
//       \ /            January 2004
//       \ /
// ______\_/_______     Contibutors: Franois Laburthe, Hadrien Cambazard, Guillaume Rochart...

package choco.palm.test;

import junit.framework.TestCase;

import choco.palm.PalmProblem;
import choco.palm.PalmSolver;
import choco.palm.explain.Explanation;
import choco.palm.search.PalmGlobalSearchSolver;
import choco.palm.prop.PalmEngine;
import choco.palm.integer.PalmIntVar;
import choco.palm.integer.PalmIntDomain;
import choco.palm.integer.constraints.*;

import java.util.ArrayList;
import java.util.Collections;

import choco.ContradictionException;
import choco.Constraint;
import choco.Solution;
import choco.Problem;
import choco.search.AbstractGlobalSearchLimit;
import choco.integer.IntVar;

public class PalmSolveTest extends TestCase {

  public void testBranchAndBoundMaximize() {
    Problem myPb = new PalmProblem();
    IntVar x = myPb.makeEnumIntVar("X", 1, 10);
    IntVar y = myPb.makeEnumIntVar("Y", 1, 10);
    IntVar z = myPb.makeEnumIntVar("Z", 1, 10);
    IntVar obj = myPb.makeEnumIntVar("OBJ", 1, 100);
    myPb.post(myPb.geq(x, z));
    myPb.post(myPb.neq(x, z));

    myPb.post(myPb.geq(z, y));
    myPb.post(myPb.neq(z, y));

    myPb.post(myPb.neq(x, y));
    myPb.post(myPb.eq(myPb.scalar(new IntVar[]{x, y, z}, new int[]{1, 1, -1}), obj));

    ((PalmProblem)myPb).maximize(obj);  // TODO
    int nbSolutions = myPb.getSolver().getNbSolutions();
    for (int i = 0; i < nbSolutions; i++) {
      Solution solution = ((PalmSolver) myPb.getSolver()).getSolution(i);
      System.out.println(solution);
    }

    System.out.println("nb Sol " + nbSolutions);

    assertEquals(9, ((PalmSolver) myPb.getSolver()).getSolution(nbSolutions - 1).getValue(3));
  }

  public void testArithmetic() {
    Problem myPb = new PalmProblem();
    IntVar x = myPb.makeEnumIntVar("X", 1, 5);
    IntVar y = myPb.makeBoundIntVar("Y", 1, 5);
    IntVar z = myPb.makeEnumIntVar("Z", 1, 5);
    Constraint A = myPb.geq(x, myPb.plus(y, 1));
    Constraint B = myPb.geq(y, myPb.plus(z, 1));
    myPb.post(A);
    myPb.post(B);

    A.setPassive();
    A.setActive();

    try {
      myPb.propagate();
    } catch (ContradictionException e) {
      assertTrue(false);
    }

    assertEquals(3, x.getInf());
    assertEquals(5, x.getSup());
    assertEquals(2, y.getInf());
    assertEquals(4, y.getSup());
    assertEquals(1, z.getInf());
    assertEquals(3, z.getSup());

    Explanation expl = ((PalmProblem)myPb).makeExplanation();
    ((PalmIntVar) x).self_explain(PalmIntDomain.INF, expl);
    assertTrue(expl.contains(A));
    assertTrue(expl.contains(B));

    ((PalmEngine) myPb.getPropagationEngine()).remove(A);

    try {
      myPb.propagate();
    } catch (ContradictionException e) {
      assertTrue(false);
    }

    assertEquals(1, x.getInf());
    assertEquals(5, x.getSup());
    assertEquals(2, y.getInf());
    assertEquals(5, y.getSup());
    assertEquals(1, z.getInf());
    assertEquals(4, z.getSup());
  }

  public void testArithmetic2() {
    Problem myPb = new PalmProblem();
    IntVar x = myPb.makeBoundIntVar("X", 1, 5);
    IntVar y = myPb.makeBoundIntVar("Y", 1, 5);
    IntVar z = myPb.makeBoundIntVar("Z", 1, 5);
    Constraint A = myPb.geq(x, myPb.plus(y, 1));
    Constraint B = myPb.geq(y, myPb.plus(z, 1));
    myPb.post(A);
    myPb.post(B);

    A.setPassive();

    try {
      myPb.propagate();
    } catch (ContradictionException e) {
      assertTrue(false);
    }

    assertEquals(1, x.getInf());
    assertEquals(5, x.getSup());

    assertEquals(2, y.getInf());
    assertEquals(5, y.getSup());

    assertEquals(1, z.getInf());
    assertEquals(4, z.getSup());
  }

  public void testSolveArithmetic() {
    Problem myPb = new PalmProblem();
    IntVar x = myPb.makeBoundIntVar("X", 1, 5);
    IntVar y = myPb.makeBoundIntVar("Y", 1, 5);
    IntVar z = myPb.makeBoundIntVar("Z", 1, 5);
    Constraint A = myPb.geq(x, myPb.plus(y, 1));
    Constraint B = myPb.geq(y, myPb.plus(z, 1));
    myPb.post(A);
    myPb.post(B);

    myPb.solveAll();

    int nbSolutions = myPb.getSolver().getNbSolutions();

    System.out.println("Solve Solutions : " + nbSolutions);
    ((PalmProblem)myPb).printRuntimeSatistics(); // TODO
    assertEquals(10, nbSolutions);
  }

  public void testSolveArithmetic2() {
    Problem myPb = new PalmProblem();
    IntVar x = myPb.makeEnumIntVar("X", 1, 5);
    IntVar y = myPb.makeBoundIntVar("Y", 1, 5);
    IntVar z = myPb.makeEnumIntVar("Z", 1, 5);
    Constraint A = myPb.eq(x, y);
    Constraint B = myPb.eq(y, z);
    myPb.post(A);
    myPb.post(B);

    myPb.solveAll();

    int nbSolutions = myPb.getSolver().getNbSolutions();

    System.out.println("Solve2 Solutions : " + nbSolutions);
    ((PalmProblem)myPb).printRuntimeSatistics();
    assertEquals(5, nbSolutions);
  }

  public void testSolveArithmetic3() {
    Problem myPb = new PalmProblem();
    IntVar x = myPb.makeEnumIntVar("X", 1, 5);
    IntVar y = myPb.makeEnumIntVar("Y", 1, 5);
    IntVar z = myPb.makeBoundIntVar("Z", 1, 5);
    Constraint A = myPb.neq(x, y);
    Constraint B = myPb.neq(y, z);
    Constraint C = myPb.neq(x, z);
    myPb.post(A);
    myPb.post(B);
    myPb.post(C);

    myPb.solveAll();

    int nbSolutions = myPb.getSolver().getNbSolutions();

    System.out.println("Solve3 Solutions : " + nbSolutions);
    ((PalmProblem)myPb).printRuntimeSatistics();
    assertEquals(60, nbSolutions);
  }

  public void testLinComb() {
    Problem myPb = new PalmProblem();
    IntVar w = myPb.makeEnumIntVar("W", 1, 5);
    IntVar x = myPb.makeBoundIntVar("X", 1, 5);
    IntVar y = myPb.makeEnumIntVar("Y", 1, 5);
    IntVar z = myPb.makeBoundIntVar("Z", 1, 5);
    Constraint A = myPb.eq(myPb.scalar(new int[]{1, 1, -1, -1}, new IntVar[]{w, x, y, z}), 1);
    myPb.post(A);

    myPb.solveAll();

    int nbSolutions = myPb.getSolver().getNbSolutions();

    System.out.println("Lin Comb Solutions : " + nbSolutions);
    ((PalmProblem)myPb).printRuntimeSatistics();
    assertEquals(80, nbSolutions);
  }

  public void testElt() {
    Problem myPb = new PalmProblem();
    IntVar index = myPb.makeEnumIntVar("IndexVar", 0, 4);
    IntVar value = myPb.makeEnumIntVar("ValueVar", 1, 10);
    Constraint A = new PalmElt(index, value, 0, new int[]{0, 2, 4, 5, 12});  //TODO : API pour poser la contrainte
    myPb.post(A);

    try {
      myPb.propagate();
    } catch (ContradictionException e) {
      assertTrue(false);
    }
    System.out.println("indexVar : " + index.getInf() + " - " + index.getSup());
    System.out.println("valueVar : " + value.getInf() + " - " + value.getSup());
    assertEquals(1, index.getInf());
    assertEquals(3, index.getSup());
    assertEquals(5, value.getSup());
    assertEquals(2, value.getInf());
    assertTrue(!value.canBeInstantiatedTo(3));

    myPb.solveAll();

    int nbSolutions = myPb.getSolver().getNbSolutions();

    for (int i = 0; i < nbSolutions; i++) {
      Solution solution = ((PalmSolver) myPb.getSolver()).getSolution(i);
      System.out.println(solution);
    }
    System.out.println("Element Solutions : " + nbSolutions);
    ((PalmProblem)myPb).printRuntimeSatistics();
    assertEquals(3, nbSolutions);
  }

  public void testElt2() {
    Problem myPb = new PalmProblem();
    IntVar index = myPb.makeEnumIntVar("IndexVar", 0, 3);
    IntVar value1 = myPb.makeEnumIntVar("ValueVar1", 0, 1);
    IntVar value2 = myPb.makeEnumIntVar("ValueVar2", 0, 1);
    IntVar value3 = myPb.makeEnumIntVar("ValueVar3", 0, 1);
    IntVar value4 = myPb.makeEnumIntVar("ValueVar4", 0, 1);
    Constraint A = new PalmElt(index, value1, 0, new int[]{1, 0, 0, 0});
    Constraint B = new PalmElt(index, value2, 0, new int[]{0, 1, 0, 0});
    Constraint C = new PalmElt(index, value3, 0, new int[]{0, 0, 1, 0});
    Constraint D = new PalmElt(index, value4, 0, new int[]{0, 0, 0, 1});
    myPb.post(A);
    myPb.post(B);
    myPb.post(C);
    myPb.post(D);

    myPb.solveAll();

    int nbSolutions = myPb.getSolver().getNbSolutions();
    for (int i = 0; i < nbSolutions; i++) {
      Solution solution = ((PalmSolver) myPb.getSolver()).getSolution(i);
      System.out.println(solution);
    }

    System.out.println("Element Solutions : " + nbSolutions);
    ((PalmProblem)myPb).printRuntimeSatistics();
    assertEquals(4, nbSolutions);
  }

  public void testElt2D() {
    Problem myPb = new PalmProblem();
    IntVar index1 = myPb.makeEnumIntVar("IndexVar1", 0, 2);
    IntVar index2 = myPb.makeEnumIntVar("IndexVar2", 0, 3);
    IntVar value = myPb.makeEnumIntVar("ValueVar", 0, 10);
    Constraint A = new PalmElt2D(index1, index2, value,
        new int[][]{{1, 2, 0}, {12, 1, 5}, {0, 6, 1}}, 3, 3);  //TODO : API pour poser la contrainte
    myPb.post(A);

    try {
      myPb.propagate();
    } catch (ContradictionException e) {
      assertTrue(false);
    }

    System.out.println("sup : " + value.getSup());
    assertEquals(6, value.getSup());
    myPb.solveAll();

    int nbSolutions = myPb.getSolver().getNbSolutions();
    for (int i = 0; i < nbSolutions; i++) {
      Solution solution = ((PalmSolver) myPb.getSolver()).getSolution(i);
      System.out.println(solution);
    }
    System.out.println("Element2D Solutions : " + nbSolutions);
    ((PalmProblem)myPb).printRuntimeSatistics();
    assertEquals(8, nbSolutions);
  }

  public void testSolveArithmeticBench() {
    Problem myPb = new PalmProblem();
    int n = 5;
    IntVar[] vars = new IntVar[n];
    for (int i = 0; i < n; i++) {
      if (i % 2 == 0)
        vars[i] = myPb.makeBoundIntVar("X" + i, 1, n + 3);
      else
        vars[i] = myPb.makeEnumIntVar("X" + i, 1, n + 3);
    }
    for (int i = 0; i < n - 1; i++) {
      IntVar var1 = vars[i];
      IntVar var2 = vars[i + 1];
      myPb.post(myPb.geq(var1, myPb.plus(var2, 1)));
    }

    myPb.solveAll();

    int nbSolutions = myPb.getSolver().getNbSolutions();
    assertEquals(56, nbSolutions);
    System.out.println("Arith Bench -- Solutions : " + nbSolutions);
    ((PalmProblem)myPb).printRuntimeSatistics();
  }

  public void testNextSolutionAPI() {
    Problem myPb = new PalmProblem();
    int n = 5;
    IntVar[] vars = new IntVar[n];
    for (int i = 0; i < n; i++) {
      if (i % 2 == 0)
        vars[i] = myPb.makeBoundIntVar("X" + i, 1, n + 3);
      else
        vars[i] = myPb.makeEnumIntVar("X" + i, 1, n + 3);
    }
    for (int i = 0; i < n - 1; i++) {
      IntVar var1 = vars[i];
      IntVar var2 = vars[i + 1];
      myPb.post(myPb.geq(var1, myPb.plus(var2, 1)));
    }

    myPb.solve();
    while(myPb.nextSolution() == Boolean.TRUE) {}

    int nbSolutions = myPb.getSolver().getNbSolutions();
    assertEquals(56, nbSolutions);
    System.out.println("Arith Bench -- Solutions : " + nbSolutions);
    ((PalmProblem)myPb).printRuntimeSatistics();
  }

  public void testComparator() {
    PalmProblem myPb = new PalmProblem();
    IntVar x = myPb.makeBoundIntVar("X", 1, 5);
    IntVar y = myPb.makeBoundIntVar("Y", 1, 5);
    IntVar z = myPb.makeBoundIntVar("Z", 1, 5);
    Constraint A = new PalmGreaterOrEqualXYC(x, y, 1);
    Constraint B = new PalmGreaterOrEqualXYC(y, z, 1);
    Constraint C = new PalmAssignment(x, 2);
    myPb.post(A);
    myPb.post(B);
    myPb.post(C, 0);

    ArrayList list = new ArrayList();
    list.add(B);
    list.add(C);
    list.add(A);

    assertEquals(C, Collections.min(list, new choco.palm.explain.BetterConstraintComparator()));
    assertEquals(A, Collections.max(list, new choco.palm.explain.BetterConstraintComparator()));
  }

  public void testLimit() {
    Problem pb = new PalmProblem();
    int n = 10;
    IntVar[] vars = new IntVar[10];
    for (int i = 0; i < vars.length; i++) {
      vars[i] = pb.makeBoundIntVar("v" + i, 1, n + 1);
      if (i > 0) {
        pb.post(pb.gt(vars[i], vars[i - 1]));
      }
    }

    pb.getSolver().setNodeLimit(2);
    pb.solveAll();
    assertEquals(2, ((AbstractGlobalSearchLimit)
        ((PalmGlobalSearchSolver) pb.getSolver().getSearchSolver()).
        getLimit(PalmGlobalSearchSolver.LIMIT_NODES)).getNbTot());
  }

}
