package choco.real.constraint;

import choco.real.RealVar;
import choco.Var;
import choco.AbstractConstraint;
import choco.AbstractVar;
import choco.bool.AbstractCompositeConstraint;

public abstract class AbstractUnRealConstraint extends AbstractConstraint implements RealConstraint {
  /**
   * The unique variable of the constraint.
   */
  protected RealVar v0;

  /**
   * The index of this constraints w.r.t. the variable v0.
   */
  protected int cIdx0;

  /**
   * Returns the only one variable if i=0, null otherwise.
   */
  public RealVar getRealVar(int i) {
    if (i == 0)
      return v0;
    else
      return null;
  }

  /**
   * Returns the number of real variables.
   */
  public int getRealVarNb() {
    return 1;
  }

  /**
   * Let v0 be the i-th var of c, records that c is the idx-th constraint involving v0.
   */
  public void setConstraintIndex(int i, int idx) {
    if (i == 0)
      cIdx0 = idx;
    else
      throw new Error("bug in setConstraintIndex i:" + i + " this: " + this);
  }

  /**
   * Returns the index of this constraints in all constraints involving v0.
   */
  public int getConstraintIdx(int idx) {
    if (idx == 0)
      return cIdx0;
    else
      return -1;
  }

  /**
   * Returns the number of variables.
   */
  public int getNbVars() {
    return 1;
  }

  /**
   * Returns the only one variable v0 if i==0, null otherwise.
   */
  public Var getVar(int i) {
    if (i == 0)
      return v0;
    else
      return null;
  }

  /**
   * Checks if the only one variable of the constraint is instantiated.
   */
  public boolean isCompletelyInstantiated() {
    return v0.isInstantiated();
  }

  public int assignIndices(AbstractCompositeConstraint root, int i) {
    int j=i;
    j++; root.connectVar((AbstractVar) v0, j);
    setConstraintIndex(0, v0.getNbConstraints() - 1);
    return j;
  }
}
