package choco.real.search;

import choco.search.AbstractBinIntBranching;
import choco.search.AbstractGlobalSearchSolver;
import choco.ContradictionException;
import choco.Problem;
import choco.real.RealVar;
import choco.real.RealMath;
import choco.branch.IntBranching;

/**
 * A binary branching assigning interval to subinterval.
 */

public class AssignInterval extends AbstractBinIntBranching implements IntBranching {
  protected RealVarSelector varSelector;

  public AssignInterval(RealVarSelector varSelector) {
    this.varSelector = varSelector;
  }

  public Object selectBranchingObject() {
    return varSelector.selectRealVar();
  }

  public boolean branchOn(Object x, int n) throws ContradictionException {
    AbstractGlobalSearchSolver algo = manager;
    Problem pb = algo.problem;
    boolean nodeSuccess = false;
    boolean nodeFinished = false;
    int i = getFirstBranch(x);
    algo.newTreeNode();
    try {
      do {
        boolean branchSuccess = false;
        try {
          pb.getPropagationEngine().checkCleanState();
          pb.getEnvironment().worldPush();
          goDownBranch(x, i);
          if (explore(n + 1)) {
            branchSuccess = true;
          }
        } catch (ContradictionException e) {
          ;
        }
        if (!branchSuccess) {
          pb.getEnvironment().worldPop();
        }
        algo.endTreeNode();
        algo.postDynamicCut();
        goUpBranch(x, i);
        if (branchSuccess) {
          nodeSuccess = true;
        }
        if (!finishedBranching(x, i)) {
          i = getNextBranch(x, i);
        } else {
          nodeFinished = true;
        }
      } while (!nodeSuccess && !nodeFinished);
    } catch (ContradictionException e) {
      nodeSuccess = false;
    }
    return nodeSuccess;
  }

  public void goDownBranch(Object x, int i) throws ContradictionException {
    super.goDownBranch(x,i);
    if (i == 1) {
      ((RealVar)x).intersect(RealMath.firstHalf((RealVar)x));
      manager.problem.propagate();
    } else if (i == 2) {
      ((RealVar)x).intersect(RealMath.secondHalf((RealVar)x));
      manager.problem.propagate();
    } else {
      logger.severe("!! Not a valid value for AssignInterval branching !!");
    }
  }

  public void goUpBranch(Object x, int i) throws ContradictionException {
    super.goUpBranch(x,i);
  }
}
