package choco.real.var;

import choco.prop.VarEvent;
import choco.ContradictionException;
import choco.Var;
import choco.AbstractVar;
import choco.Constraint;
import choco.integer.IntConstraint;
import choco.util.BitSet;
import choco.util.IntIterator;
import choco.real.RealVar;
import choco.real.constraint.RealListener;

/**
 * An event for real interval variable modifications.
 */
public class RealVarEvent extends VarEvent {
  public static final int INCINF = 0;
  public static final int DECSUP = 1;

  public final static int EMPTYEVENT = 0;
  public final static int BOUNDSEVENT = 3;
  public final static int INFEVENT = 1;
  public final static int SUPEVENT = 2;

  public RealVarEvent(AbstractVar var) {
    super(var);
  }

  public String toString() {
    return ("VarEvt(" + modifiedVar.toString() + ")[" + eventType + ":"
        + (BitSet.getBit(eventType, INCINF) ? "I" : "")
        + (BitSet.getBit(eventType, DECSUP) ? "S" : "")
        + "]");
  }

  public void clear() {
    this.eventType = EMPTYEVENT;
    ((RealVar) modifiedVar).getDomain().clearDeltaDomain();
  }

  protected boolean release() {
    return ((RealVar) modifiedVar).getDomain().releaseDeltaDomain();
  }

  protected void freeze() {
    ((RealVar) modifiedVar).getDomain().freezeDeltaDomain();
    cause = NOEVENT;
    eventType = 0;
  }

  public boolean getReleased() {
    return ((RealVar) modifiedVar).getDomain().getReleasedDeltaDomain();
  }

  public boolean propagateEvent() throws ContradictionException {
    logger.finer("propagate " + this.toString());
    // first, mark event
    int evtType = eventType;
    int evtCause = cause;
    freeze();

    if (evtType <= BOUNDSEVENT) {     // only two first bits (bounds) are on
      if (evtType == INFEVENT)
        propagateInfEvent(evtCause);
      else if (evtType == SUPEVENT)
        propagateSupEvent(evtCause);
      else if (evtType == BOUNDSEVENT) {
        propagateBoundsEvent(evtCause);
      }
    }
    // last, release event
    return release();
  }

  /**
   * Propagates the update to the upper bound
   */
  public void propagateSupEvent(int evtCause) throws ContradictionException {
    AbstractVar v = getModifiedVar();
    Constraint[] constraints = v.getConstraints();
    Integer[] indices = v.getVarIndices();

    for (IntIterator cit = getAllListenersButIterator(evtCause); cit.hasNext(); ) {
      int index = cit.next();
      ((RealListener) constraints[index]).awakeOnSup(indices[index].intValue());
    }
  }

  /**
   * Propagates the update to the lower bound
   */
  public void propagateInfEvent(int evtCause) throws ContradictionException {
    AbstractVar v = getModifiedVar();
    Constraint[] constraints = v.getConstraints();
    Integer[] indices = v.getVarIndices();

    for (IntIterator cit = getAllListenersButIterator(evtCause); cit.hasNext(); ) {
      int index = cit.next();
      ((RealListener) constraints[index]).awakeOnInf(indices[index].intValue());
    }
  }

  /**
   * Propagates the update to the domain lower and upper bounds
   */
  public void propagateBoundsEvent(int evtCause) throws ContradictionException {
    AbstractVar v = getModifiedVar();
    Constraint[] constraints = v.getConstraints();
    Integer[] indices = v.getVarIndices();

    for (IntIterator cit = getAllListenersButIterator(evtCause); cit.hasNext(); ) {
      int index = cit.next();
      ((RealListener) constraints[index]).awakeOnInf(indices[index].intValue());
      ((RealListener) constraints[index]).awakeOnSup(indices[index].intValue());
    }
  }

  /**
   * Retrieves the event type
   */
  public int getEventType() { /// TODO : dans VarEvent !
    return eventType;
  }
}
