package choco.set;

import choco.Problem;
import choco.Constraint;
import choco.integer.IntVar;
import choco.set.var.SetVarImpl;
import choco.set.constraint.*;

// **************************************************
// *                   J-CHOCO                      *
// *   Copyright (C) F. Laburthe, 1999-2003         *
// **************************************************
// *  an open-source Constraint Programming Kernel  *
// *     for Research and Education                 *
// **************************************************

public class SetModeler {

  /**
   * The problem this modler should build classes for.
   */
  protected Problem problem;

  /**
   * Builds a modeler for the specified problem.
   */
  public SetModeler(Problem problem) {
    this.problem = problem;
  }

  public SetVar makeSetVar(String name, int a, int b) {
    return problem.makeSetVar(name,a,b);
  }

  public SetVar makeSetVar(int a, int b) {
    return problem.makeSetVar("",a,b);
  }

  /**
   * Enforce a set to be the intersection of two others.
   * @param inter the inersection of sv1 and sv2
   * @return
   */
  public Constraint setInter(SetVar sv1, SetVar sv2, SetVar inter) {
    return new SetIntersection(sv1,sv2,inter);
  }

  public Constraint eqCard(SetVar sv, IntVar v) {
     return new SetCard(sv,v,true,true);
  }

  public Constraint eqCard(SetVar sv, int val) {
     return new SetCard(sv,problem.makeConstantIntVar("cste" + sv,val),true,true);
  }

  public Constraint geqCard(SetVar sv, IntVar v) {
     return new SetCard(sv,v,false,true);
  }

  public Constraint geqCard(SetVar sv, int val) {
     return new SetCard(sv,problem.makeConstantIntVar("cste" + sv,val),false,true);
  }

  public Constraint leqCard(SetVar sv, IntVar v) {
     return new SetCard(sv,v,true,false);
  }

  public Constraint leqCard(SetVar sv, int val) {
     return new SetCard(sv,problem.makeConstantIntVar("cste" + sv,val),true,false);
  }

  public Constraint setDisjoint(SetVar sv1, SetVar sv2) {
    return new Disjoint(sv1,sv2);
  }

  public Constraint member(int val, SetVar sv1) {
    return new MemberX(sv1,val);
  }

  public Constraint member(SetVar sv1, int val) {
    return new MemberX(sv1,val);
  }

  public Constraint member(SetVar sv1, IntVar var) {
    return new MemberXY(sv1,var);
  }

  public Constraint member(IntVar var, SetVar sv1) {
    return new MemberXY(sv1,var);
  }

  public Constraint notMember(int val, SetVar sv1) {
    return new NotMemberX(sv1,val);
  }

  public Constraint notMember(SetVar sv1, int val) {
    return new NotMemberX(sv1,val);
  }

  public Constraint notMember(SetVar sv1, IntVar var) {
    return new NotMemberXY(sv1,var);
  }

  public Constraint notMember(IntVar var, SetVar sv1) {
    return new NotMemberXY(sv1,var);
  }


}
