// **************************************************
// *                   J-CHOCO                      *
// *   Copyright (C) F. Laburthe, 1999-2003         *
// **************************************************
// *  an open-source Constraint Programming Kernel  *
// *     for Research and Education                 *
// **************************************************

/* File choco.test.integer.IntVarEventTest.java, last modified by FLABURTHE 2 janv. 2004 */
package choco.test.integer;

import choco.Constraint;
import choco.ContradictionException;
import choco.Problem;
import choco.AbstractVar;
import choco.util.IntIterator;
import choco.integer.constraints.AbstractBinIntConstraint;
import choco.integer.var.IntDomainVar;
import choco.integer.IntVar;
import choco.integer.IntConstraint;
import choco.integer.constraints.EqualXYC;
import choco.integer.var.IntVarEvent;
import choco.prop.ChocEngine;
import choco.prop.PropagationEvent;
import junit.framework.TestCase;

import java.util.HashSet;
import java.util.logging.Logger;

public class IntVarEventTest extends TestCase {
  private Logger logger = Logger.getLogger("choco.test.integer.IntVarEventTest");
  private Problem pb;
  private ChocEngine pe;
  private choco.integer.var.IntDomainVar x;
  private choco.integer.var.IntDomainVar y;
  private choco.integer.var.IntDomainVar z;
  private Constraint c1;
  private Constraint c2;
  private Constraint c3;
  PropagationEvent evt;

  class LocalConstraintClass extends AbstractBinIntConstraint {
    public boolean isSatisfied() {
      return false;
    }

    public void propagate() {
    }

    public void awakeOnInf(int idx) {
    }

    public void awakeOnSup(int idx) {
    }

    public LocalConstraintClass(choco.integer.IntVar x, choco.integer.IntVar y) {
      v0 = x;
      v1 = y;
    }
  }

  protected void setUp() {
    logger.fine("IntVarEvent Testing...");
    pb = new Problem();
    pe = pb.getChocEngine();
    x = (choco.integer.var.IntDomainVar) pb.makeBoundIntVar("X", 0, 100);
    y = (choco.integer.var.IntDomainVar) pb.makeBoundIntVar("Y", 0, 100);
    z = (choco.integer.var.IntDomainVar) pb.makeBoundIntVar("Z", 0, 100);
    c1 = new LocalConstraintClass(x, y);
    c2 = new LocalConstraintClass(y, z);
    c3 = new LocalConstraintClass(z, x);
  }

 protected void tearDown() {
    c1 = null;
    c2 = null;
    c3 = null;
    x = null;
    y = null;
    z = null;
    pb = null;
    pe = null;
  }

  public void test1() {
    assertEquals(0, pe.getNbPendingEvents());
    pb.post(c1);
    pb.post(c2);
    assertEquals(2, pe.getNbPendingEvents());
    HashSet expectedSet = new HashSet();
    expectedSet.add(c1);
    expectedSet.add(c2);
    HashSet tmp = new HashSet();
    tmp.add(pe.getPendingEvent(0).getModifiedObject());
    tmp.add(pe.getPendingEvent(1).getModifiedObject());
    assertEquals(expectedSet, tmp);
    try {
       pb.propagate();
    } catch (ContradictionException e) {
      assertFalse(true);
    }
    assertEquals(0, pe.getNbPendingEvents());
    x.getDomain().updateInf(1);
    pe.postUpdateInf(x, 1);
    y.getDomain().updateSup(95);
    pe.postUpdateSup(y, 1);
    y.getDomain().updateInf(3);
    pe.postUpdateInf(y, 0); // and not a value above 1, such as 2 !!

    assertEquals(2, pe.getNbPendingEvents());
    evt = pe.getPendingEvent(0);
    assertEquals(evt.getModifiedObject(), x);
    assertEquals(IntVarEvent.INCINFbitvector, ((IntVarEvent) evt).getEventType());
    assertEquals(1, ((IntVarEvent) evt).getCause());

    evt = pe.getPendingEvent(1);
    assertEquals(evt.getModifiedObject(), y);
    assertEquals(IntVarEvent.BOUNDSbitvector, ((IntVarEvent) evt).getEventType());
    assertEquals(-1, ((IntVarEvent) evt).getCause());
  }

/**
 * tests that a bound event on a variable with two constraints and no cause, yields two propagation steps
 */
  public void test2() {
    c1 = new EqualXYC(x, y, 2);
    c2 = new EqualXYC(y, z, 1);
    assertEquals(0, pe.getNbPendingEvents());
    pb.post(c1);
    pb.post(c2);
    assertEquals(2, pe.getNbPendingEvents());
    HashSet expectedSet = new HashSet();
    expectedSet.add(c1);
    expectedSet.add(c2);
    HashSet tmp = new HashSet();
    tmp.add(pe.getPendingEvent(0).getModifiedObject());
    tmp.add(pe.getPendingEvent(1).getModifiedObject());
    assertEquals(expectedSet, tmp);
    try {
       pb.propagate();
    } catch (ContradictionException e) {
      assertFalse(true);
    }
    assertEquals(0, pe.getNbPendingEvents());

    y.getDomain().updateSup(90);
    pe.postUpdateSup(y, 0);
    y.getDomain().updateInf(10);
    pe.postUpdateInf(y, 1);
    assertEquals(1, pe.getNbPendingEvents());
    evt = pe.getPendingEvent(0);
    assertEquals(evt.getModifiedObject(), y);
    assertEquals(IntVarEvent.BOUNDSbitvector, ((IntVarEvent) evt).getEventType());


    IntIterator cit = ((IntVarEvent) evt).getAllListenersButIterator(((IntVarEvent) evt).getCause());
    assertTrue(cit.hasNext());
    assertEquals(0,cit.next());
    assertTrue(cit.hasNext());
    assertEquals(1,cit.next());
  }

}
