package chocosamples;

import choco.*;
import choco.real.RealExp;
import choco.real.RealVar;
import choco.real.RealModeler;
import choco.real.RealInterval;
import choco.real.search.AssignInterval;
import choco.real.search.CyclicRealVarSelector;
import choco.real.constraint.Equation;
import choco.real.exp.*;

import java.util.logging.Logger;
import java.util.logging.Level;
import java.util.logging.ConsoleHandler;
import java.util.logging.Handler;
import java.util.List;

public class CycloHexan {
  public static void main(String[] args) {
    System.out.println(new CycloHexan().demo());
  }

  public String demo() {
    Problem pb = new Problem();
    pb.setPrecision(1e-8);
    RealModeler m = new RealModeler(pb);

    RealVar x = m.makeRealVar("x");
    RealVar y = m.makeRealVar("y", -1.0e8, 1.0e8);
    RealVar z = m.makeRealVar("z", -1.0e8, 1.0e8);

    RealExp exp1 = m.plus(m.mult(m.power(y, 2), m.plus(m.cst(1.0), m.power(z, 2))),
        m.mult(z, m.minus(z, m.mult(m.cst(24), y))));

    RealExp exp2 = m.plus(m.mult(m.power(z, 2), m.plus(m.cst(1.0), m.power(x, 2))),
        m.mult(x, m.minus(x, m.mult(m.cst(24), z))));

    RealExp exp3 = m.plus(m.mult(m.power(x, 2), m.plus(m.cst(1.0), m.power(y, 2))),
        m.mult(y, m.minus(y, m.mult(m.cst(24), x))));

    Equation eq1 = (Equation) m.eq(exp1, m.cst(-13));
    eq1.addBoxedVar(y);
    eq1.addBoxedVar(z);
    Equation eq2 = (Equation) m.eq(exp2, m.cst(-13));
    eq2.addBoxedVar(x);
    eq2.addBoxedVar(z);
    Equation eq3 = (Equation) m.eq(exp3, m.cst(-13));
    eq3.addBoxedVar(x);
    eq3.addBoxedVar(y);

    pb.post(eq1); pb.post(eq2); pb.post(eq3);

    /*Logger.getLogger("choco.search").setLevel(Level.ALL);
    Handler h = new ConsoleHandler();
    h.setLevel(Level.ALL);
    Logger.getLogger("choco").addHandler(h);

    Logger.getLogger("choco.search").fine("test !");*/

    boolean first = false;
    Solver solver = pb.getSolver();
    solver.setFirstSolution(first);
    solver.generateSearchSolver(pb);
    solver.addGoal(new AssignInterval(new CyclicRealVarSelector(pb)));
    solver.launch();

    packSolutions(pb);
    StringBuffer s = new StringBuffer();
    List solutions = solver.getSearchSolver().solutions;
    s.append("The CycloHexane problem consists in finding the 3D configuration of a cyclohexane molecule." +
        "It is decribed with a system of three non linear equations : \n" +
        " y^2 * (1 + z^2) + z * (z - 24 * y) = -13 \n" +
        " x^2 * (1 + y^2) + y * (y - 24 * x) = -13 \n" +
        " z^2 * (1 + x^2) + x * (x - 24 * z) = -13 \n" +
        "It has been taken from the Elisa project (LINA) examples. \n \n");
    s.append(solutions.size() + " solutions : \n");
    for (int i = 0; i < solutions.size(); i++) {
      Solution solution = (Solution) solutions.get(i);
      for(int v = 0; v < pb.getNbRealVars(); v++) {
        s.append("var nb " + v + " in " + solution.getRealValue(v) + "\n");
      }
      s.append("\n");
    }
    return s.toString();
  }

  public static void packSolutions(Problem pb) {
    Solver solver = pb.getSolver();
    List solus = solver.getSearchSolver().solutions;

    for (int i = 0; i < solus.size(); i++) {
      Solution sol = (Solution) solus.get(i);
      for (int j = 0; j < i; j++) {
        Solution prev = (Solution) solus.get(j);
        boolean ok = true;
        for(int v = 0; v < pb.getNbRealVars(); v++) {
          RealInterval inter1 = sol.getRealValue(v);
          RealInterval inter2 = prev.getRealValue(v);
          double inf = Math.min(inter1.getInf(), inter2.getInf());
          double sup = Math.max(inter1.getSup(), inter2.getSup());
          if ((sup - inf) > pb.getPrecision() * 10) {ok = false; break;}
        }
        if (ok) {
          for(int v = 0; v < pb.getNbRealVars(); v++) {
            RealInterval inter1 = sol.getRealValue(v);
            RealInterval inter2 = prev.getRealValue(v);
            double inf = Math.min(inter1.getInf(), inter2.getInf());
            double sup = Math.max(inter1.getSup(), inter2.getSup());
            prev.recordRealValue(v, new RealIntervalConstant(inf, sup));
          }
          solus.remove(sol);
          i--;
          break;
        }
      }
    }
  }
}
